/*
 * main.c
 *
 *  Created on: May 10, 2014
 *      Author: Miguel Angel Sanchez Muñoz
 */

/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "semphr.h"
#include "task.h"

#include "LPC17xx.h"

#include "type.h"
#include "gpio.h"
#include "led.h"
#include "motor.h"
#include "delay.h"
#include "debug.h"

#include "config.h"

/* Demo includes. */
#include "basic_io.h"

/* Used as a loop counter to create a very crude delay. */
#define mainDELAY_LOOP_COUNT		( 0xfffff )

xQueueHandle xQueue = NULL;

xSemaphoreHandle xMutexConnection;

xTaskHandle xTaskMotorDefaultHandle;
xTaskHandle xTaskMotorGiroHandle;
xTaskHandle xTaskMotorBackHandle;
xTaskHandle xTaskConnectionHandle;
xTaskHandle xTaskWatchDogHandle;
xTaskHandle xTaskSendDataHandle;


/* The task functions. */
static void vTaskMotorDefault( void *pvParameters );
static void vTaskMotorGiro( void *pvParameters );
static void vTaskMotorBack( void *pvParameters );
static void vTaskConnection( void *pvParameters );
static void vTaskWatchDogConnection( void *pvParameters );
static void vTaskSendData( void *pvParameters );
static void vTaskRun( void *pvParameters );
/*-----------------------------------------------------------*/


void EINT0_IRQHandler(void) {

	LPC_SC->EXTINT = 0x00000001;

    vTaskResume(xTaskMotorGiroHandle);
}

void EINT1_IRQHandler(void) {

    LPC_SC->EXTINT = 0x00000002;

    vTaskResume(xTaskMotorBackHandle);
}

int main( void )
{

	Motor_Init(STEEP, RESET, DIR_RIGHT, DIR_LEFT);

	initEINT0();
	initEINT1();

	xMutexConnection = xSemaphoreCreateMutex();
	xSemaphoreTake(xMutexConnection, portMAX_DELAY);

	xQueue = xQueueCreate( 10, 8 );

	#ifdef DEBUG_ON
	debug_INIT(DebugPort, DebugBaudrate);
	debug("START DEBUG");
	#endif

	if( xQueue != NULL && xMutexConnection != NULL)
	{
		xTaskCreate( vTaskMotorDefault, "Task Motor", 240, NULL, 1, &xTaskMotorDefaultHandle );

		xTaskCreate( vTaskMotorGiro, "Task Giro", 240, NULL, 1, &xTaskMotorGiroHandle );

		xTaskCreate( vTaskMotorBack, "Task Back", 240, NULL, 1, &xTaskMotorBackHandle );

		xTaskCreate( vTaskConnection, "Task Connection", 240, NULL, 1, &xTaskConnectionHandle );

		xTaskCreate( vTaskWatchDogConnection, "Task WatchDog Connection", 240, NULL, 1, &xTaskWatchDogHandle );

		xTaskCreate( vTaskSendData, "Task Send Data", 240, NULL, 1, &xTaskSendDataHandle );

		xTaskCreate( vTaskRun, "Task Run", 240, NULL, 1, NULL );

		/* Start the scheduler so our tasks start executing. */
		vTaskStartScheduler();
	}

	for( ;; );
	return 0;
}
/*-----------------------------------------------------------*/


static void vTaskMotorDefault( void *pvParameters )
{

	Motor_Stop();

	xQueueSendToBack( xQueue, &"MT:GO", portMAX_DELAY );

	/* As per most tasks, this task is implemented in an infinite loop. */
	for( ;; )
	{
		Motor_Go();

		Motor_Start();

		vTaskSuspendAll();
		Motor_Steep(GO_SPEED);
		xTaskResumeAll();

	}
}
/*-----------------------------------------------------------*/

static void vTaskMotorGiro( void *pvParameters )
{

vTaskSuspend( NULL );

	for( ;; )
	{
		vTaskSuspend( xTaskMotorDefaultHandle );

		xQueueSendToBack( xQueue, &"MT:GIRO", portMAX_DELAY );

		Motor_Stop();

		Motor_DirRandom();

		delayMs(500);

		Motor_Start();

		uint32_t i;
		for ( i = 0; i < 250; i++ ){
			vTaskSuspendAll();
			Motor_Steep(GIRO_SPEED);
			xTaskResumeAll();
		}

		Motor_Stop();

		xQueueSendToBack( xQueue, &"MT:GO", portMAX_DELAY );

		vTaskResume( xTaskMotorDefaultHandle );

		vTaskSuspend( NULL );
	}
}
/*-----------------------------------------------------------*/

static void vTaskMotorBack( void *pvParameters )
{

vTaskSuspend( NULL );

	for( ;; )
	{
		vTaskSuspend( xTaskMotorDefaultHandle );

		xQueueSendToBack( xQueue, &"MT:BACK", portMAX_DELAY );

		Motor_Stop();
		Motor_Back();
		delayMs(500);
		Motor_Start();

		uint32_t i;
		for ( i = 0; i < 1000; i++ ){
			vTaskSuspendAll();
			Motor_Steep(BACK_SPEED);
			xTaskResumeAll();
		}

		Motor_Stop();

		vTaskResume( xTaskMotorGiroHandle );

		vTaskSuspend( NULL );
	}
}
/*-----------------------------------------------------------*/


static void vTaskConnection( void *pvParameters )
{

	for( ;; )
		{

		#ifdef DEBUG_ON
			debug("CONFIG WIFLY PORT");
		#endif
		Wifly_INIT(WiflyPort, WiflyBaudrate);


		#ifdef DEBUG_ON
			debug("CONFIG WIFLY WLAN");
		#endif
		if (Wifly_WlanConnection(SSID, WLAN_MODE, WLAN_KEY))
		{
			#ifdef DEBUG_ON
			debug("CONFIG WIFLY WLAN OK");
			#endif

			#ifdef DEBUG_ON
				debug("CONNECT TO SERVER");
			#endif
			if (Wifly_OpenHost(IP_REMOTE, PORT_REMOTE, TIME_AUTOCONN))
			{
				#ifdef DEBUG_ON
					debug("CONNECT TO SERVER OK");
				#endif

				xSemaphoreGive(xMutexConnection);
			}
		}

		if(xTaskIsTaskSuspended(xTaskWatchDogHandle) == pdTRUE) {
			vTaskResume( xTaskWatchDogHandle );
		}

		vTaskSuspend( NULL );

	}

}
/*-----------------------------------------------------------*/

static void vTaskWatchDogConnection( void *pvParameters )
{
portTickType xLastWakeTime;
const portTickType xTicksToWait = 30000 / portTICK_RATE_MS;

vTaskSuspend( NULL );

#ifdef DEBUG_ON
	debug("START WATCHDOG CONNECTION");
#endif

xLastWakeTime = xTaskGetTickCount();

	for( ;; )
	{
		vTaskDelayUntil(&xLastWakeTime, xTicksToWait);

		if (!Wifly_SendDataHost("alive"))
			{
				#ifdef DEBUG_ON
					debug("NOT CONNECTION RECONECT");
				#endif

				vTaskResume( xTaskConnectionHandle );

			} else
			{
				#ifdef DEBUG_ON
					debug("HOST ALIVE");
				#endif
			}
	}
}
/*-----------------------------------------------------------*/

static void vTaskSendData( void *pvParameters )
{
#ifdef DEBUG_ON
	debug("START SEND DATA");
#endif

/* Declare the variable that will hold the values received from the queue. */
int8_t QueueData;
portBASE_TYPE xStatus;
const portTickType xTicksToWait = 100 / portTICK_RATE_MS;
char buffer[BUFSIZE];

	/* This task is also defined within an infinite loop. */
	for( ;; )
	{

		xStatus = xQueueReceive( xQueue, &QueueData, xTicksToWait );

		if( xStatus == pdPASS )
		{
			xSemaphoreTake(xMutexConnection, portMAX_DELAY);

			#ifdef DEBUG_ON
				sprintf(buffer, "Queue data: %s", (char *)&QueueData);
				debug(buffer);
			#endif

			Wifly_SendDataHost(&QueueData);

			xSemaphoreGive(xMutexConnection);
		}
	}

	return;
}
/*-----------------------------------------------------------*/


static void vTaskRun( void *pvParameters )
{

led_init(LED_RUN);

	/* As per most tasks, this task is implemented in an infinite loop. */
	for( ;; )
	{
		led_toggle();
		delayMsPassive(500);
	}
}
/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}


